USE AdventureWorks2014;
SET NOCOUNT ON;
GO

-- show in Plan Explorer
-- what happens when you use unsargeable search criteria?

-- index scan
SELECT COUNT(*) 
  FROM Sales.SalesOrderHeader
  WHERE YEAR(OrderDate) = 2005 
    AND MONTH(OrderDate) = 7;

-- index scan
SELECT COUNT(*) 
  FROM Sales.SalesOrderHeader
  WHERE CONVERT(CHAR(6), OrderDate, 112) = '200507';

-- index scan
SELECT COUNT(*)
  FROM Sales.SalesOrderHeader
  WHERE DATEDIFF(MONTH, '20050701', OrderDate) = 0;

-- much more efficient index seek
SELECT COUNT(*) 
  FROM Sales.SalesOrderHeader
  WHERE OrderDate >= '20050701'
    AND OrderDate < '20050801';

-- ALWAYS use an open-ended range without 
-- applying any formula or calculation to columns.



-- NEVER use shorthand

-- this works ok:
DECLARE @dt DATETIME = GETDATE();
SELECT @dt + 1;
GO
-- this breaks:
DECLARE @d DATE = GETDATE();
SELECT @d + 1;
GO
-- always use explicit DATEADD



-- POP QUIZ: what will these yield?

DECLARE @today DATE = '20140412';

SELECT DATEPART(w, @today), DATEPART(m, @today), DATEPART(y, @today);
/*
       (a) 15           (a) 4            (a) 14
       (b) 7            (b) 0            (b) 2014
       (c) 16           (c) 3            (c) 102
*/
GO













-- ALWAYS spell it out. If you mean MONTH, type MONTH.
-- You write the query once, so the typing is negligible.
-- Other people have to read and understand this query.




-- what happens when you try to find the "end" of a period?

DECLARE @start DATETIME, @end DATETIME;

SELECT 
  @start = DATEADD(DAY, DATEDIFF(DAY, 1, CURRENT_TIMESTAMP), 0),
  @end   = DATEADD(MILLISECOND, -3, DATEADD(DAY, 1, @start));

SELECT start = @start, 
       [Datetime] = @end;

SELECT start = @start, 
       [Smalldatetime] = CONVERT(SMALLDATETIME, @end); -- rounds up

SELECT start = @start, 
       [Date] = CONVERT(DATE, @end); -- rounds down

SELECT start = @start, 
       [Datetime2] = CONVERT(DATETIME2(7), @end); -- could theoretically miss data
GO
-- ALWAYS use >= start of range and < DATEADD(DAY, 1, end of range)






-- date formats. Try the below queries after changing these settings around.
SET LANGUAGE FRENCH;
SET LANGUAGE ENGLISH;
SET LANGUAGE BRITISH;
SET DATEFORMAT YDM;
SET DATEFORMAT MDY;

-- unsafe:

SELECT CONVERT(DATETIME, '2012-09-08');
SELECT CONVERT(DATETIME, '09/08/2012');
SELECT CONVERT(DATETIME, '2012-09-08 12:34:56.789');

-- safe:

SELECT CONVERT(DATETIME, '20120908');
SELECT CONVERT(DATETIME, '2012-09-08T12:34:56.789');

-- always safe but only for DATE data type:

SELECT CONVERT(DATE, '2012-09-08');
SELECT CONVERT(DATE, '20120908');