USE AdventureWorks2012;
SET NOCOUNT ON;
GO

CREATE PROCEDURE dbo.KitchenSink_Typical
	@LastName     NVARCHAR(50) = NULL,
	@StoreID      INT = NULL,
	@CustomerID   INT = NULL,
	@MinimumSpend MONEY = NULL
AS
BEGIN
  SET NOCOUNT ON;

  SELECT p.FirstName, p.LastName, c.CustomerID
    FROM Sales.Customer AS c
    INNER JOIN Person.Person AS p
    ON c.CustomerID = p.BusinessENtityID
    WHERE 
      (p.LastName LIKE @LastName OR @LastName IS NULL)
    AND 
      (c.StoreID = @StoreID OR @StoreID IS NULL)
    AND
      (c.CustomerID = COALESCE(@CustomerID, c.CustomerID))
    AND 
    (
      @MinimumSpend IS NULL OR
	  (
	    @MinimumSpend IS NOT NULL
        AND EXISTS
        (
          SELECT 1 FROM Sales.SalesOrderHeader
            WHERE CustomerID = c.CustomerID
            GROUP BY CustomerID
            HAVING SUM(TotalDue) >= @MinimumSpend
        )
	  )
    );-- OPTION (RECOMPILE);
END
GO

CREATE PROCEDURE dbo.KitchenSink_DynamicSQL
	@LastName     NVARCHAR(50) = NULL,
	@StoreID      INT = NULL,
	@CustomerID   INT = NULL,
	@MinimumSpend MONEY = NULL
AS
BEGIN
  SET NOCOUNT ON;

  DECLARE @sql NVARCHAR(MAX) = N'
  SELECT p.FirstName, p.LastName, c.CustomerID
    FROM Sales.Customer AS c
    INNER JOIN Person.Person AS p
    ON c.CustomerID = p.BusinessENtityID
    WHERE 1 = 1';

  IF @LastName IS NOT NULL
    SET @sql += N' AND p.LastName LIKE @LastName';

  IF @StoreID IS NOT NULL
    SET @sql += N' AND c.StoreID = @StoreID';

  IF @CustomerID IS NOT NULL
    SET @sql += N' AND c.CustomerID = @CustomerID';

  IF @MinimumSpend IS NOT NULL
    SET @sql += N' AND EXISTS
        (
          SELECT 1 FROM Sales.SalesOrderHeader
            WHERE CustomerID = c.CustomerID
            GROUP BY CustomerID
            HAVING SUM(TotalDue) >= @MinimumSpend
        )';

  --SET @sql += N' OPTION (RECOMPILE);'

  EXEC sp_executesql @sql,
    N'@LastName NVARCHAR(50), @StoreID INT,
        @CustomerID INT, @MinimumSpend MONEY',
	@LastName, @StoreID, @CustomerID, @MinimumSpend;
END
GO

-- look in Plan Explorer:

EXEC dbo.KitchenSink_Typical @MinimumSpend = 10000;
EXEC dbo.KitchenSink_Typical @LastName = N'Er%';
EXEC dbo.KitchenSink_Typical @LastName = N'Z%';

EXEC dbo.KitchenSink_DynamicSQL @MinimumSpend = 10000;
EXEC dbo.KitchenSink_DynamicSQL @LastName = N'Er%';
EXEC dbo.KitchenSink_DynamicSQL @LastName = N'Z%';


DROP PROCEDURE 
  dbo.KitchenSink_Typical, 
  dbo.KitchenSink_DynamicSQL;