-- qs_fn_stripdownchars: Required for above SQL.
if exists	(select	*
             from sysobjects
             where id = object_id(N'qs_fn_stripdownchars')
               and OBJECTPROPERTY(id,	N'IsScalarFunction')	=	1)
  drop function qs_fn_stripdownchars
go

create function qs_fn_stripdownchars
(	@s	nvarchar(4000), @nShowPlacements int = 0)
returns nvarchar(4000)
as
begin
	declare	@s2	nvarchar(4000)
	declare @c	nchar(2)
	declare @p	int
	set		@p	= 0
	set		@s2	= ''
	declare @pComment	int
	set		@pComment = 1

	declare @sComment1			sysname
	declare @sComment2			sysname
	declare @sNumber			sysname
	declare @sConstantString	sysname
	if (@nShowPlacements = 0)
		select	@sComment1	= ''
			,	@sComment2	= ''
			,	@sNumber	= ''
			,	@sConstantString	= ''
	else
		select	@sComment1		= '<comment1>'
			,	@sComment2		= '<comment2>'
			,	@sNumber		= '<number>'
			,	@sConstantString= '<constant-string>'

	declare @nMaxLoops	int		-- Defensive programming to avoid infinite looping (as the conditions for looping are odd)
	set		@nMaxLoops = 100
	while @nMaxLoops>0
	begin
		set @nMaxLoops = @nMaxLoops - 1
		select	@p	= patindex('%[0-9'']%',@s)
		if (@p is null or @p = 0)
			break

		begin
			select	@pComment = charindex('--',@s)	-- From the Previous Value
			if @pComment < @p and @pComment>0
				set @p = @pComment
			
			select	@pComment = charindex('/*',@s)	-- From the Previous Value
			if @pComment < @p and @pComment>0
				set @p = @pComment
		end

		set	@c = substring(@s,@p,2)
		if (substring(@c,1,1) = '''')	-- Is A Quote
		begin
			set @s2 = @s2 + substring(@s,1,@p-1) + @sConstantString  	-- Copy the fixed chars.
			set	@p	= charindex('''',@s,@p+1)
			set @s	= substring(@s,@p+1,4000)	-- Reassign @s to AFTER the next Quote.
		end
		else if (@c = '--')
		begin
			set @s2 = @s2 + substring(@s,1,@p-1) + @sComment1	-- Copy the fixed chars.
			set @p  = charindex(convert(nchar(1),0x0D),@s,@p+1)	-- Up to the newline!
			if (@p = 0)
				set @p = len(@s)
			set @s	= substring(@s,@p+1,4000)	-- Reassign @s to AFTER the next Quote.
		end
		else if (@c = '/*')
		begin
			set @s2 = @s2 + substring(@s,1,@p-1) + @sComment2  	-- Copy the fixed chars.
			set @p  = charindex('*/',@s,@p+1)
			if (@p = 0)
				set @p = len(@s)
			set @s	= substring(@s,@p+2,4000)	-- Reassign @s to AFTER the next Quote.
		end
		else
		begin
			set @s2 = @s2 + substring(@s,1,@p-1) + @sNumber	-- Copy the fixed chars.
			set @s	= substring(@s,@p+1,4000)	
			set @s	= substring(@s,patindex('%[^0-9]%',@s), 4000)	--  Reassign @s to AFTER the next Non-number.
		end
	end

	return @s2 + @s
end
go 



-- SQL 2005: How many sessions are running Ad-hoc SQL that could be sharing the same plans?
select	db_name(y.dbid) DBName
	,	count(*) nSessions
	,	count(distinct y.sql_handle)	distinct_handles
	,	sum(y.waittime) waittime
	,	sum(y.waittime)/count(*) WaittimePerSimilarSQL
	,	y.thesql_clean
	,	max(y.open_tran)	open_tran
from	(
	select	x.sql_handle
		,	x.dbid
		,	x.waittime
		,	(dbo.qs_fn_stripdownchars((sql.text),0))	thesql_clean
		,	x.open_tran
		,	x.spid
		,	x.blocked
	from	master.dbo.sysprocesses	x
	cross apply sys.dm_exec_sql_text(x.sql_handle) sql
	where	x.spid>50
)	y
group by y.dbid, y.thesql_clean
order by nSessions desc;

-- SQL: Potential Memory saving moving to Prepared SQL Commnands.
select	count(*)	SimilarCacheEntries
	,	(sum(pagesused))*8	TotalKB
	,	(sum(pagesused)-avg(pagesused))*8	PotentialKBSaving
	,	dbo.qs_fn_stripdownchars((c.sql),0)	thesql
into	#t
from	master.dbo.syscacheobjects	c
where	c.objtype = 'Adhoc'
group by dbo.qs_fn_stripdownchars((c.sql),0)
order by SimilarCacheEntries desc

select	*
from	#t

select	sum(PotentialKBSaving)/1024.0	TotalPotentialMBSaving
	,	sum(TotalKB)/1024.0 TotalAdhocMB
from	#t

drop table #t;

-- SQL: Moving Cache size, and Adhoc percentage.
set nocount on
declare @t table (dts datetime, AdhocSizeMB numeric(20,10), CacheSizeMB numeric(20,10))

declare @nDuration int
set		@nDuration = 60
declare @dts datetime
set		@dts = getdate()
while	getdate() < dateadd(second,@nDuration,@dts)
begin
	insert into @t (dts,AdhocSizeMB,CacheSizeMB)
	select	getdate()
		,	sum(case when objtype = 'Adhoc' then pagesused else 0 end)*(8/1024.0)	AdhocSizeMB
		,	sum(pagesused)*(8/1024.0)	CacheSizeMB
	from	master.dbo.syscacheobjects

	waitfor delay '00:00:02'
end
set nocount off
select	*
	,	convert(numeric(10,2),(100.0*AdhocSizeMB)/CacheSizeMB)		AdhocSQLPercent
from	@t;


