-- space (total, used, and free) in tempdb
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT SUM(user_object_reserved_page_count
+ internal_object_reserved_page_count
+ version_store_reserved_page_count
+ mixed_extent_page_count
+ unallocated_extent_page_count) * (8.0/1024)
AS [TotalSizeOfTempDB(MB)]
, SUM(user_object_reserved_page_count
+ internal_object_reserved_page_count
+ version_store_reserved_page_count
+ mixed_extent_page_count) * (8.0/1024)
AS [UsedSpace (MB)]
, SUM(unallocated_extent_page_count * (8.0/1024))
AS [FreeSpace (MB)]
FROM sys.dm_db_file_space_usage;

-- tempdb total space usage by object type
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT
SUM (user_object_reserved_page_count) * (8.0/1024)
AS [User Objects (MB)],
SUM (internal_object_reserved_page_count) * (8.0/1024)
AS [Internal Objects (MB)],
SUM (version_store_reserved_page_count) * (8.0/1024)
AS [Version Store (MB)],
SUM (mixed_extent_page_count)* (8.0/1024)
AS [Mixed Extent (MB)],
SUM (unallocated_extent_page_count)* (8.0/1024)
AS [Unallocated (MB)]
FROM sys.dm_db_file_space_usage;


--Space usage by session in tempdb
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT es.session_id
, ec.connection_id
, es.login_name
, es.host_name
, st.text
, su.user_objects_alloc_page_count
, su.user_objects_dealloc_page_count
, su.internal_objects_alloc_page_count
, su.internal_objects_dealloc_page_count
, ec.last_read
, ec.last_write
, es.program_name
FROM sys.dm_db_session_space_usage su
INNER JOIN sys.dm_exec_sessions es
ON su.session_id = es.session_id
LEFT OUTER JOIN sys.dm_exec_connections ec
ON su.session_id = ec.most_recent_session_id
OUTER APPLY sys.dm_exec_sql_text(ec.most_recent_sql_handle) st
WHERE su.session_id > 50;