USE AdventureWorks2014;
SET NOCOUNT ON;
GO

-- CIX scan:
SELECT BusinessEntityID, PersonType, FirstName, LastName
  FROM Person.Person
  WHERE LastName LIKE N'S%';

-- IX scan + key lookup:
SELECT BusinessEntityID, PersonType, FirstName, LastName
  FROM Person.Person
  WHERE LastName LIKE N'%ch%';


-- Unfortunately, if we parameterize this, 
-- we may be stuck with the same plan for
-- both cases - recompile may help but then
-- you pay compile costs. Let's test this.

DBCC FREEPROCCACHE;
DBCC FREESYSTEMCACHE('ALL');
GO

ALTER PROCEDURE dbo.SearchPerson_Typical
  @LastName NVARCHAR(50)
AS
BEGIN
  SET NOCOUNT ON;

  SELECT BusinessEntityID, PersonType, FirstName, LastName
    FROM Person.Person
    WHERE LastName LIKE @LastName;
END
GO

-- look at these in Plan Explorer:

EXEC dbo.SearchPerson_Typical @LastName = N'%ch%';
EXEC dbo.SearchPerson_Typical @LastName = N'S%';
-- same plan - seek + key lookup, not optimal for both queries
GO

CREATE PROCEDURE dbo.SearchPerson_DynamicSQL
  @LastName NVARCHAR(50)
AS
BEGIN
  SET NOCOUNT ON;

  DECLARE @sql NVARCHAR(MAX) = N'SELECT BusinessEntityID, PersonType, FirstName, LastName
    FROM Person.Person /* ' + CASE WHEN @LastName LIKE N'[%]%' THEN 'scan' ELSE 'seek' END
	+ ' */ WHERE LastName LIKE @LastName;';

  PRINT @sql;
  EXEC sp_executesql @sql, N'@LastName NVARCHAR(50)', @LastName;
END
GO

EXEC dbo.SearchPerson_DynamicSQL @LastName = N'%ch%';
EXEC dbo.SearchPerson_DynamicSQL @LastName = N'S%';

GO

DROP PROCEDURE 
  dbo.SearchPerson_Typical, 
  dbo.KitchenSink_DynamicSQL;