DBCC FREEPROCCACHE;
DBCC TRACEON(3604);
GO
-- Initial memo contents
SELECT
    p.Name,
    Total = SUM(inv.Quantity)
FROM Production.Product AS p
JOIN Production.ProductInventory AS inv ON
    inv.ProductID = p.ProductID
WHERE
    p.Name LIKE N'[A-G]%'
GROUP BY
    p.Name
OPTION (QUERYTRACEON 8608);

SELECT
    p.Name,
    Total = SUM(inv.Quantity)
FROM Production.Product AS p
JOIN Production.ProductInventory AS inv ON
    inv.ProductID = p.ProductID
WHERE
    p.Name LIKE N'[A-G]%'
GROUP BY
    p.Name
OPTION (RECOMPILE, QUERYTRACEON 8615);

DBCC TRACEOFF(3604);

/*
If we want to see which rules were used when optimizing a query, one option is to use an 
undocumented DMV which shows the number of times a rule has been asked to estimate how 
valuable it might be in the current context (its promise value), the number of times 
the rule was used to generate an alternative section of the tree (built), and the 
number of times the output of the rule was successfully incorporated into the 
search space (succeeded). A sample of the contents of the sys.dm_exec_query_transform-
ation_stats DMV is shown below: 
*/

SELECT *
FROM sys.dm_exec_query_transformation_stats;
--Undocumented DMV

/*
By taking a snapshot view of this information before and after optimizing a particular 
query, we can see which rules were run. The DMV is instance-wide however, so you would 
need to run these tests on a system to which you have exclusive access.   
*/

SELECT *
FROM sys.dm_exec_query_optimizer_info;